import cPickle
import zipfile
import tarfile
import os

from wxPython.wx import *
from wxPython.lib.anchors import LayoutAnchors
from wxPython.lib.scrolledpanel import wxScrolledPanel
from wxPython.glcanvas import *

name = 'Open from an archive'

_repeat = ''

def getSprite(parent, repeat=0):
    global _repeat
    
    if not repeat or _repeat == '':
        dlg = wxFileDialog(parent, 'Choose archive', os.getcwd(),
                           '', 'Archives (*.zip)|*.zip', wxOPEN)
        if dlg.ShowModal() == wxID_OK:
            filename = dlg.GetPath()
            _repeat = filename
            arc = Archive()
            arc.open(filename, 'r')
            namelist = arc.namelist()
            arcdlg = xzvfArchiveContent(parent, arc)
            if arcdlg.ShowModal() == wxID_OK:
                dlg.Destroy()
                num = arcdlg.getNum()
                arcdlg.Destroy()
                return arc.getObject(namelist[num])
            arcdlg.Destroy()
        dlg.Destroy()
    else:
        arc = Archive()
        arc.open(_repeat, 'r')
        namelist = arc.namelist()
        arcdlg = xzvfArchiveContent(parent, arc)
        if arcdlg.ShowModal() == wxID_OK:
            num = arcdlg.getNum()
            arcdlg.Destroy()
            return arc.getObject(namelist[num])
        arcdlg.Destroy()
    return None

class xzvfArchiveVirtualList(wxListCtrl):
    def __init__(self, parent, id, arc):
        wxListCtrl.__init__(self, parent, id, wxPoint(10, 10), size=wxSize(480, 300),
                            style = wxLC_REPORT | wxLC_VIRTUAL | wxSUNKEN_BORDER)

        self.arc = arc

        self.InsertColumn(0, 'Name')
        self.SetColumnWidth(0, 450)
        
        self.SetItemCount(self.arc.getNumItems())

    def OnGetItemText(self, item, col):
        return self.arc.getName(item)

class xzvfArchiveContent(wxDialog):
    def __init__(self, parent, archive, size=wxSize(500, 340)):
        wxDialog.__init__(self, parent, -1, 'Choose file', pos=wxDefaultPosition,
                          size=size, style=wxDEFAULT_DIALOG_STYLE)
        self.sizer = wxBoxSizer(wxVERTICAL)

        self.liste = xzvfArchiveVirtualList(self, 12, archive)
        EVT_LIST_ITEM_SELECTED(self, 12, self.OnSelect)

        btn = wxButton(self, wxID_OK, ' OK ', wxPoint(10, 320))
        btn.SetDefault()

        self.SetAutoLayout(1)
        self.n = -1

    def OnSelect(self, event):
        self.n = event.m_itemIndex
        

    def getNum(self):
        return self.n
            
class Archive:
    def open(self, filename, mode='a', zip=1):
        self.type = zip
        if zip:
            self.zip = zipfile.ZipFile(filename, mode, zipfile.ZIP_DEFLATED)
        else:
            self.tar = tarfile.open(filename, mode)
            
    def close(self):
        if self.type:
            self.zip.close()
        else:
            self.tar.close()

    def addString(self, bytes, arcname):
        if self.type:
            fp = file('.temp', 'w')
            fp.write(bytes)
            fp.close()
            self.addFile('.temp', arcname)
            os.remove('.temp')
#            self.zip.writestr(arcname, bytes)

    def addFile(self, filename, arcname=''):
        if self.type:
            if arcname == '':
                self.zip.write(filename)
            else:
                self.zip.write(filename, arcname)
        else:
            if arcname == '':
                self.tar.add(filename)
            else:
                self.tar.add(filename, arcname)

    def namelist(self):
        if self.type:
            return self.zip.namelist()
        else:
            return self.tar.list(False)

    def getNumItems(self):
        return len(self.namelist())

    def getName(self, i):
        return (self.namelist())[i]

    def addObject(self, object, arcname=''):
        filename='.temporaryfile'
        fp = file(filename, 'wb')
        cPickle.dump(object, fp, -1)
        fp.close()
        self.addFile(filename, arcname)
        os.remove(filename)

    def getFile(self, filename):
        if self.type:
            return self.zip.read(filename)
        else:
            return self.tar.extractfile(filename).read()

    def getObject(self, filename):
        return cPickle.loads(self.getFile(filename))

