from OpenGL.GL import *
from OpenGL.GLU import *
import Image
import decode
#import compose
import struct

def _i16(c):
	return ord(c[0]) + (ord(c[1])<<8)

def _i32(c):
	return ord(c[0]) + (ord(c[1])<<8) + (ord(c[2])<<16) + (ord(c[3])<<24)

def _s32(n):
	return chr(n % 256) + chr((n >> 8) % 256) + chr((n >> 16) % 256) + chr((n >> 32) % 256)

#def _getInt(fp):
#	return _i32(fp.read(4)) 

#def _getShort(fp):
#	return _i16(fp.read(2))


def _getInt(fp):
	return struct.unpack('<i', fp.read(4))[0]#_i32(fp.read(4)) 

def _getShort(fp):
	return struct.unpack('<h', fp.read(2))[0]#_i32(fp.read(4)) 

class Spr:
	def __init__(self, filename):
		fp = file(filename, 'rb')
		self.getPal(fp)
		fp.seek(2, 1)
		compressed = ord(fp.read(2)[0])
		self.images = []
		self.nImages = _getShort(fp)
		self.nRgbaImages = _getShort(fp)
		self.rgbaImages = []
		
		for i in range(self.nImages):
			sizeX = _getShort(fp)
			sizeY = _getShort(fp)
			if compressed:
				compLen = _getShort(fp)
				compImage = fp.read(compLen)
				indexImage = decode.rleDecode(sizeX*sizeY, compImage)
			else:
				indexImage = fp.read(sizeX*sizeY)
			lrev = list(indexImage)
			lrev.reverse()
			self.images.append((sizeX, sizeY, ''.join(lrev)))

		for i in range(self.nRgbaImages):
			sizeX = _getShort(fp)
			sizeY = _getShort(fp)
			rgbaImage = fp.read(sizeX*sizeY*4)
			temp = ''
			for k in range(sizeX):
				for l in range(sizeY):
					lrev = list(rgbaImage[(k*sizeY+l)*4:(k*sizeY+l+1)*4])
					lrev.reverse()
					temp += ''.join(lrev)
			self.rgbaImages.append((sizeX, sizeY, temp))
			
		fp.close()

	def rleDecode(self, compImage, compLen):
		i = 0
		image = ''
		while i < compLen:
			if ord(compImage[i]) == 0:
				i += 1
				image += chr(0)*ord(compImage[i])
				i += 1
			else:
				image += compImage[i]
				i += 1
		return image

	def rgbDecode(self, sizeX, sizeY, indexImage):
		return decode.rgbDecode(indexImage, self.pal)

	def rgbaDecode(self, sizeX, sizeY, indexImage):
		rgba = decode.rgbaDecode(indexImage, self.pal)
		return rgba

	def toPILImages(self):
		pilImages = []
		for i in range(self.nImages):
			sizeX, sizeY, indexImage = self.images[i]
			data = self.rgbDecode(sizeX, sizeY, indexImage)
			pilImages.append(Image.fromstring('RGB', (sizeX, sizeY), data))
		return pilImages

	def getPal(self, fp):
		pos = fp.tell()
		fp.seek(-1024, 2)
		self.pal = fp.read(1024)
		fp.seek(pos, 0)
		
	def toGLTextures(self):
		maxx = 0
		maxy = 0

	#	glTextures = []
		if self.nImages == 1:
			glTextures = [glGenTextures(self.nImages)]
		else:
			glTextures = glGenTextures(self.nImages)

		for i in range(self.nImages):
			sizeX, sizeY, indexImage = self.images[i]
			indexImage = decode.upsidedown(sizeX, sizeY, indexImage)
			rawData = self.rgbaDecode(sizeX, sizeY, indexImage)
			newSize = bestSize(sizeX), bestSize(sizeY)
			if newSize != (sizeX, sizeY):
				image = Image.fromstring('RGBA', (sizeX, sizeY)
							 ,rawData)
				image = image.resize(newSize, Image.NEAREST)
				data = image.tostring("raw", 'RGBA')
			else:
				data = rawData
			
			glBindTexture(GL_TEXTURE_2D, glTextures[i])

			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, newSize[0],
				     newSize[1], 0, GL_RGBA, GL_UNSIGNED_BYTE,
				     data)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
					GL_NEAREST)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
					GL_NEAREST)
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,
				  GL_MODULATE)
		#	glTextures.append((newSize[0], newSize[1], data))
		return glTextures

def bestSize( dim ):
	"""Try to figure out the best power-of-2 size for the given dimension

	At the moment, this is the next-largest power-of-two
	which is also <= glGetInteger( GL_MAX_TEXTURE_SIZE ).
	"""
	boundary = min( (glGetInteger( GL_MAX_TEXTURE_SIZE ), dim))
	test = 1
	while test < boundary:
		test = test * 2
	return test
